%
%   SUNTRK.M  Set parameters and perform simulation of a Simulink model
%             for a sun tracking servo mechanism (with proportional control)
% 
%   See file LTSERVO.M for a similar simulation (normal Matlab modeling)
%
%   File prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all,   close all
%
%   MOTOR CHARACTERISTICS (usually given with spec sheet or measured)
%   Ra - Resistance of the motor armature (ohms)
%   La - Inductance of the motor armature (H)
%   kv - velocity constant (Vsec/rad)
%   kt - torque constant (Nm/A)
      Ra = 1.75;      La = 2.83e-3;      kv = 0.093;      kt = 0.0924;
%
%   SYSTEM CHARACTERISTICS (study sensitivity of system dynamics to these parameters)
%   Jeq - Inertia seen by the motor.  Includes inertia of the load. (kg m^2)
%   Beq - Mechanical damping coefficient associated with rotation. (Nms)
%   K   - Gain of photodiode circuit (volts/radian)
%
      nfig = 0;
      repeat = 'y';
      while repeat == 'y'
%----------------------------------------------------------------
%   MAIN SENSITIVITY ANALYSIS (repeat as many times as desired)
%----------------------------------------------------------------
      Jeq = 30.0e-6;      Beq = 5.0e-3;      K = 25.0;
      suntrksl      % show Simulink model of sun tracking system
      n = menu('Select variable to analyze','GAIN Parameter','DAMPING Term', ...
               'INERTIA of System');  
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT GAINS
      if n == 1
         opt = 'Sensitivity to Gain'
         K = input('Input low gain [typical = 25 V/radian]: ');
         [A1,B1,C1,D1] = linmod('suntrksl');   sys1 = ss(A1,B1,C1,D1);
         K = input('Input nominal gain [typical = 25 V/radian]: ');
         [A2,B2,C2,D2] = linmod('suntrksl');   sys2 = ss(A2,B2,C2,D2);
         K = input('Input high gain [typical = 25 V/radian]: ');
         [A3,B3,C3,D3] = linmod('suntrksl');   sys3 = ss(A3,B3,C3,D3);
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT MECHANICAL DAMPING
      elseif n == 2
         opt = 'Sensitivity to Damping'
         Beq = input('Input low damping factor [typical = 5.0e-3 Nms]: ');
         [A1,B1,C1,D1] = linmod('suntrksl');   sys1 = ss(A1,B1,C1,D1);
         Beq = input('Input nominal damping factor [typical = 5.0e-3 Nms]: ');
         [A2,B2,C2,D2] = linmod('suntrksl');   sys2 = ss(A2,B2,C2,D2);
         Beq = input('Input high damping factor [typical = 5.0e-3 Nms]: ');
         [A3,B3,C3,D3] = linmod('suntrksl');   sys3 = ss(A3,B3,C3,D3);
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT INERTIAS
      elseif n == 3
         opt = 'Sensitivity to Inertia'
         Jeq = input('Input low inertia [typical = 30.0e-6 kg m^2]: ');
         [A1,B1,C1,D1] = linmod('suntrksl');   sys1 = ss(A1,B1,C1,D1);
         Jeq = input('Input nominal inertia [typical = 30.0e-6 kg m^2]: ');
         [A2,B2,C2,D2] = linmod('suntrksl');   sys2 = ss(A2,B2,C2,D2);
         Jeq = input('Input high inertia [typical = 30.0e-6 kg m^2]: ');
         [A3,B3,C3,D3] = linmod('suntrksl');   sys3 = ss(A3,B3,C3,D3);
      end
%
%----------------------------------------------------------------  
%   GENERATE BODE PLOTS  and  STEP RESPONSE
%----------------------------------------------------------------  
%
%   Get range of frequency       
        w1 = input('Select beginning freq. exponent (rad/sec) [1]: ');
        if isempty(w1);   w1=1;   end
        w2 = input('Select end freq. exponent (rad/sec) [3]: ');
        if isempty(w2);   w2=3;   end
        w = logspace(w1,w2,100);
%
%   Bode plots of closed loop system
        figure(1)
        bode(sys1,'r',sys2,'g--',sys3,'b:',w),grid
        legend('low','mid','high','Location','SouthWest');
        hd = findobj(gcf,'Color','r'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        hd = findobj(gcf,'LineStyle','--'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        hd = findobj(gcf,'Color','b'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        title(['Frequency Response -- ', opt]);
%
%   Step response of closed loop system
        t2 = input('Specify end time (sec) [0.08]: ');
        if isempty(t2);    t2=0.08;     end
        t = linspace(0,t2,101);
        xs(:,1) = step(sys1,t);
        xs(:,2) = step(sys2,t);
        xs(:,3) = step(sys3,t);
%
%   plot step response
        figure(2)
        plot(t,xs(:,1),'r-',t,xs(:,2),'g--',t,xs(:,3),'b:','LineWidth',2),grid
        xlabel('time (sec)'),ylabel('position (radians)');
        title(['Step Response -- ', opt]);
        legend('low','mid','high');
%
        repeat = input('Do again? y/n [n]: ','s');
        if isempty(repeat);   repeat = 'n';    end
      end      % end of while repeat
%
%   normal termination